/* Implementation of PopUpList class
 *
 * Copyright (C)  1993  The Board of Trustees of  
 * The Leland Stanford Junior University.  All Rights Reserved.
 *
 * Authors: Scott Francis, Paul Kunz, and Libing Wang (SLAC)
 *          Mike Kienenberger (Alaska)
 *
 * This file is part of an Objective-C class library for a window system
 *
 * PopUpList.m,v 1.40 1995/01/06 22:12:47 pfkeb Exp
 */
 
#include "PopUpList.h"

#include "Menu.h"
#include <objc/List.h>
#include "MenuCell.h"
#include "Matrix.h"
#include <string.h>
#include <stdlib.h>

extern char *PopUpListInstanceName(void);

@interface PopUpList(WidgetSet)
- _initFrame;
- _realizeNewItem;
- _managedBy:parent wid:(void*)widget;
@end

@implementation PopUpList:Menu

- init
{
    [super init];
    instancename = PopUpListInstanceName();
    _menu_type = 0;

    return self;
}

- initFrame:(const NXRect *)aFrame
{
 /*
  * This method is not in NeXTSTEP.   Since our popuplist handles a popup
  * widget, it behaves like a control. 
  */

    [super init];

    frame.origin.x    = aFrame->origin.x;
    frame.origin.y    = aFrame->origin.y;
    frame.size.width  = aFrame->size.width;
    frame.size.height = aFrame->size.height;
    [self _initFrame];
    _menu_type = 0;
    return self;
}

- setAction:(SEL)aSelector
{    
    [matrix setAction:aSelector];
    return self;
}

- (SEL)action
{
    return [matrix action];
}

- setTarget:anObject
{
    [matrix setTarget:anObject];
    return self;
}
- target
{
    return [matrix target];
}
- selectCell:aCell;
{
    [matrix selectCell:aCell];
    return self;
}
- (const char *)selectedItem
{
    MenuCell	*cell = [matrix selectedCell];
    return [cell stringValue];
}

- addItem:(const char *)aString andRealize:(BOOL)flag
{
    MenuCell	*cell;
    int		i;

    i = [self indexOfItem:aString];
    if ( i >= 0 ) {
        return [[matrix cellList] objectAt:i];
    }
    [matrix addRow];
    cell = [[matrix cellList] lastObject];
    [cell setStringValue:aString];
    [cell _setControlView:matrix];
    if (flag)
     	[self _realizeNewItem];
    return cell;
}

- addItem:(const char *)aTitle
{    
    return [self addItem:aTitle andRealize:YES];
}

- insertItem:(const char *)aString at:(unsigned int)index
{
    List	*cellList = [matrix cellList];
    MenuCell	*aCell;
    int		i;
    
    i = [self indexOfItem:aString];
    if ( i >= 0 ) {
        aCell = [cellList removeObjectAt:i];
	[cellList insertObject:aCell at:index];
    } else {
        [matrix insertRowAt:index];
	aCell = [[matrix cellList] objectAt:index];
	[aCell setStringValue:aString];
    }
    [self _realizeNewItem];
    return aCell;
}

- removeItemAt:(unsigned int)index
{
    List	*cellList = [matrix cellList];
    id item;

    item = [cellList objectAt:index];
    [matrix removeRowAt:index andFree:YES];
    if (widgetid)
     	[self _realizeNewItem];

    return item;
}

- _managedBy:parent
{
    return [self _managedBy:parent wid:[parent _widget]];
}

- (int)indexOfItem:(const char *)aString
{
    List	*cellList = [matrix cellList];
    MenuCell	*aCell;
    int		i;
    
    i = [cellList count];
    while( i-- ) {
        aCell = [cellList objectAt:i];
	if ( !strcmp([aCell title], aString) ) {
	    return i;
	}
    }
    return (-1);
}

- read:(TypedStream *)ts
{
    [super read:ts];
#ifdef DEBUG    
    fprintf(stderr, "reading PopUpList\n");
#endif
    return self;
}

- awake
{
  /* PopUpList will be initialized by the Button control */
// [super awake];  
  return self;
}

- awakeFrame:(const NXRect *)aFrame
{
    List	*cellList = [matrix cellList];
    MenuCell	*cell;
    NXSize	size;
    int		i;

 /*
  * This method is not in NeXTSTEP.   Since our popuplist handles a popup
  * widget, it behaves like a control. 
  */

    [super awake];
    instancename = PopUpListInstanceName();
    /* need to be removed at some date */
    frame.origin.x    = aFrame->origin.x;
    frame.origin.y    = aFrame->origin.y;
    frame.size.width  = aFrame->size.width;
    frame.size.height = aFrame->size.height;
    size = frame.size;
    size.width -= 26;	// cells are smaller than popup frame
    size.height -= 2;
    _menu_type = 0;
    i = [cellList count];
    while (i-- ) {
	cell = [cellList objectAt:i];
	[cell _setSize:&size];
    }
    [matrix setCellSize:&size];
    [self _initFrame];
    return self;
}

@end
