/* X/Motif WidgetSet category implementation of  Cell class
 *
 * Copyright (C)  1994  The Board of Trustees of  
 * The Leland Stanford Junior University.  All Rights Reserved.
 *
 * Authors: Scott Francis, Fred Harris, Paul Kunz,
 *          Imran Qureshi, and Libing Wang
 * At: Stanford Linear Accelerator Center, Stanford University and
 *     University of Hawaii
 *
 * This file is part of an Objective-C class library for X/Motif
 *
 * xmCell.m,v 1.7 1995/02/21 00:31:34 pfkeb Exp
 */

#include "Cell.h"
#include "xtCell.h"

#include <stdlib.h>
#include "Motif.h"
#include <Xm/PushB.h>
#define Window X11Window
#include <X11/IntrinsicP.h>
#undef Window
#include <stdio.h>
extern char *xtCellInstanceName(void);

char *CellInstanceName(void)
{
    return xtCellInstanceName();
}

/* Private methods of Motif interface */

@implementation Cell(WidgetSet)

- _moveTo:(NXCoord)x :(NXCoord)y
{
    XtMoveWidget(widgetid, x, y);
    return self;
}

- _setEnabled:(BOOL)flag
{
    [self _setArg:XmNsensitive to:flag];
    return self;
}

- _setFrame:(const NXRect *)frame
{
    _frame.origin.x    = frame->origin.x;
    _frame.origin.y    = frame->origin.y;
    _frame.size.width  = frame->size.width;
    _frame.size.height = frame->size.height;
    [self _setArg:XmNx      to:_frame.origin.x];
    [self _setArg:XmNy      to:_frame.origin.y];
    [self _setArg:XmNwidth  to:_frame.size.width];
    [self _setArg:XmNheight to:_frame.size.height];
    return self;
}

- _setSize:(const NXSize *)size
{
    _frame.size.width = size->width;
    _frame.size.height = size->height;
    [self _setArg:XmNwidth  to:_frame.size.width];
    [self _setArg:XmNheight to:_frame.size.height];
    [self _setArg:XmNrecomputeSize to:False];
    return self;
}
- _getFrame:(NXRect *)frame
{
    [self _getArg:XmNx 	    into:(XtArgVal *)&frame->origin.x];
    [self _getArg:XmNy	    into:(XtArgVal *)&frame->origin.y];
    [self _getArg:XmNwidth  into:(XtArgVal *)&frame->size.width];
    [self _getArg:XmNheight into:(XtArgVal *)&frame->size.height];
    return self;
}
- _addCallback
{
    return self;
}

- _managedBy:parent
{
    Widget widparent;

    if (!(widparent = [parent _widget]))
        if ( ![parent respondsTo:@selector(superview)] ||
              !(widparent = 
	         [[parent perform:@selector(superview)] _widget] ) ) {
            fprintf(stderr, "Cell: parent widget not found\n");
            return 0;
        }
    [self _managedBy:parent wid:widparent];

    return self;
}    

- _managedBy:parent wid:(void *)widget;
{
    if (!classname) {
        fprintf(stderr, "Cell classname unknown\n");
    }
    if (widgetid) {
        XtManageChild(widgetid);
    } else {
	if ( !contents ) {
	  /* if no contents yet, then make default */
//	    [self setStringValue:" "];
/* I don't think that the initial value should be set to a <space>.
   People will click in the uninitialized
   cell and type something in.  The cursor will be placed after the <space>,
   and the entry will then have a <space> prepended.  This is nasty for
   filenames! E.g. " temp.tmp" is a badly formed filename.
   */
	    [self setStringValue:""];
	}
	widgetid = XtCreateManagedWidget(instancename, classname,
				     widget, arglist, numargs);
	[self _addCallback];
    }	
    parentid = parent;
    return self;
}

- _destroy
{
    if ( widgetid ) {
        XtDestroyWidget(widgetid);
    }
    widgetid = NULL;
    return self;
}    
- (void *)_widget
{
    return widgetid;
}

- _copy
{
  /* On entry, receiving Cell has copy of pointers to objects from
   * another cell.   These must be cleared so new objects can be 
   * created
   */
    allocedargs = numargs = 0;
    arglist = NULL;
    widgetid = NULL;
    return self;
}
@end
