/* WidgetSet category mplementation of View class for X/Motif
 *
 * Copyright (C)  1994  The Board of Trustees of  
 * The Leland Stanford Junior University.  All Rights Reserved.
 *
 * Authors: Scott Francis, Paul Kunz, Tom Pavel, 
 *	    Imran Qureshi, and Libing Wang (SLAC)
 *	    Adam Fedor (U Colorado)
 *
 * This file is part of an Objective-C class library for X/Motif
 *
 * xmView.m,v 1.14 1995/02/23 21:40:22 pfkeb Exp
 */

 
#include "View.h"
#include "xtResponder.h"

/* Required for implementation: */
#include "appkit/stdmacros.h"
#include <objc/List.h>
#include "appkit/Motif.h"
#include "appkit/xttools.h"	/* for XWPing */
#include "Application.h"
#include "Window.h"
#include <Xm/BulletinB.h>  /* for xmScrolledWindowWidgetClass */
#define Window X11Window	/* To avoid problems with Objective C */
#include <X11/IntrinsicP.h>
#undef Window
#include <X11/StringDefs.h>	/* For non-Motif resource names */
#include "dpsclient/xwfriends.h"

GC _createGC(XWContext context, Widget widget);
GC _copyGC(XWContext context, Widget widget, GC source);

char *ViewInstanceName(void)
{
    return "View";
}

/* Private functions we use from window */
@interface Window(WindowPrivate)
- (Pixmap)_backingPixmap;
@end

@implementation View(WidgetSet)

- _moveTo:(NXCoord)x :(NXCoord)y
{
    if ( widgetid ) {
        XtMoveWidget(widgetid, x, y );
    }
    return self;
}

- _sizeTo:(NXCoord)width :(NXCoord)height
{
    [self _setArg:XmNwidth  to:width];
    [self _setArg:XmNheight to:height];
    return self;
}

- makeNXEvent:(NXEvent *)nx fromXEvent:(XEvent *)event
/*
	converts various fields to an NXEvent-type struct, that 
	actually is not quite an NXEvent, but has many of the same
	fields to ease the transition from NeXT to X app.

 */
{
    nx->flags = 0;
    nx->type = event->type;
    nx->location.x = event->xbutton.x;
//    nx->location.y = bounds.size.height - event->xbutton.y;
    nx->location.y = event->xbutton.y;
    nx->time = (long)event->xbutton.time;
    if (event->xbutton.state & ShiftMask) {
    	nx->flags |= SHIFT_MASK;
    }
    if (event->xbutton.state & ControlMask) {
    	nx->flags |= CONTROL_MASK;
    }
    
    // these if statements can be extended to handle a few more keys...
    
//    nx->flags |= event->xbutton.state;

    // Expose events
    // The way this data is stored is a bit contrived, but  I guess I'd
    // rather do that then change the event structure. - adam
    if (event->type == Expose) {
	XExposeEvent *ex = (XExposeEvent *)event;
    	nx->location.x = ex->x;
	nx->location.y = ex->y;
	nx->data.compound.subtype = ex->count;
	nx->data.compound.misc.L[0] = ex->width;
	nx->data.compound.misc.L[1] = ex->height;
    }
    return self;
}

- _init
{
  /* This method is invoked from from the -initFrame and -awake
   * methods.  Subclasses of View may override this method to 
   * add to their initialization.
   */

    classname = xmBulletinBoardWidgetClass;
	[self _setFrame:&frame];
    [self _addArg:XmNallowOverlap:True];
    [self _addArg:XmNnoResize:True];
    [self _addArg:XmNresizePolicy :XmRESIZE_NONE];
    [self _addArg:XmNmarginWidth :0];
    [self _addArg:XmNmarginHeight :0];

/*
    [self _setArg:XmNforeground 
	to:xtGrayToPixel((XMotifContext)[NXApp context], 0.0)];
    [self _setArg:XmNbackground 
	to:xtGrayToPixel((XMotifContext)[NXApp context], 0.6666)];
*/
    return self;
}

- _getFrame
{
    [self _getArg:XmNx      into:(XtArgVal *)&frame.origin.x];
    [self _getArg:XmNy      into:(XtArgVal *)&frame.origin.y];
    [self _getArg:XmNwidth  into:(XtArgVal *)&frame.size.width];
    [self _getArg:XmNheight into:(XtArgVal *)&frame.size.height];
    return self;
}

- _managedBy:parent wid:(void*)widget
{    
    if (!classname)
       fprintf(stderr, "%s: classname unknown\n", [self name]);
    if (widgetid) {
        XtManageChild(widgetid);
    } else {
	widgetid = XtCreateManagedWidget(instancename, classname,
				         widget, [self _arglist], 
					 [self _numargs]);
	[self _addCallback];
    }
    [self _setnumargs:0];
    [self _manageChildren];
    return self;
}
 
- _unManage
{
    if ( widgetid ) {
	XtUnmanageChild(widgetid);
    }
    return self;
}
- _destroy
{
    if ( widgetid ) {
        XtDestroyWidget(widgetid);
    }
    widgetid = NULL;
    if (nil != subviews)  [subviews makeObjectsPerform:@selector(_destroy)];

    return self;
}

/* Routines to handle gstates.  We need to create a gstate for every view
   in X-Windows, because every view draws under a different widget.
   However, a separate X11 Graphic Context (GC) is not created for
   every view unless one is specifically asked for via the allocGState
   method.  Otherwise, the X11 GC is just a pointer to the superview's
   GC.  This causes changes to the X11 GC to propogate up the chain,
   but it shouldn't cause too many problems since I think this is what
   happens in DPS anyway.
*/
/* Creates a gstate. Sets some parameters to a default state.  */
- (XWgstate)_createGState
{
    XMotifContext context;
    XWgstate	  view;

    context = (XMotifContext)[NXApp context];

    NX_MALLOC(view, XWgstate_t, 1);
    view->scalex = 1.0;
    view->scaley = 1.0;
    view->rotation = 0;
    view->widget = [self _widget];

    /* Set the Drawable we are supposed to draw too.  If the window we
       are in is using backing store, we need to draw to the backing pixmap
       and then transfer the data later on.
       Note that the coordinate system for backing pixmaps is the Window
       coordinate system, not the View coordinates.
    */
    view->draw = [window _backingPixmap];
    if (view->draw == 0) {
	if (view->widget)
	    view->draw = XtWindow(view->widget);
    	memcpy(&view->cliprect, &bounds, sizeof(NXRect));
    	view->origin.x = 0;
    	view->origin.y = 0;
    } else {
	// FIXME: Need to convert to window coordinates
	NXRect cliprect;
    	cliprect = frame;
    	memcpy(&view->cliprect, &cliprect, sizeof(NXRect));
	view->origin.x = NX_X(&frame);
	view->origin.y = NX_Y(&frame);
    }
    view->update = 0;

    if ((!context->gstate || vFlags2.wantsGState) && !vFlags.validGState) {
	if (context->gstate && context->gstate->xgstate)
	    view->xgstate = _copyGC(context, view->widget, 
		context->gstate->xgstate);
	else
	    view->xgstate = _createGC(context, view->widget);
	if (view->xgstate) {
	    vFlags2.wantsGState = NO;
	    vFlags.validGState = YES;
	    // Save this gstate
    	    _gstate = view;
    	}
    } else if (context->gstate)
	view->xgstate = context->gstate->xgstate;

    if (!view->xgstate)
	fprintf(stderr, "Error (View): Couldn't create GC for gstate\n");
    return view;
}

/* Update a gstate from another gstate.  Used primarily when we a locking
   focus and need to create a gstate using the gstate of our superview.
*/
- (XWgstate)_updateGState:(XWgstate)view fromGState:(XWgstate)superg
{
    NXRect superclip;

    /* Set the Drawable we are supposed to draw too.  If the window we
       are in is using backing store, we need to draw to the backing pixmap
       and then transfer the data later on.
       We need to do this everytime, in case we resized or moved.
    */
    view->draw = [window _backingPixmap];
    memcpy(&superclip, &superg->cliprect, sizeof(NXRect));
    if (view->draw == 0) {
	if (view->widget)
	    view->draw = XtWindow(view->widget);
    	view->origin.x = 0;
    	view->origin.y = 0;
	// FIXME: convert superclip to our coordinate system
    } else {
	// FIXME: Need to convert to window coordinates
	view->origin.x = NX_X(&frame);
	view->origin.y = NX_Y(&frame);
    }
    view->update = 0;

    /* Update the gstate . We do weird stuff with NXRect because the gstate
	cliprect is actually an XWRect.
    */
    if (superg) {
    	NXRect cliprect;
        memcpy(&cliprect, &view->cliprect, sizeof(NXRect));
	NXIntersectionRect(&superclip, &cliprect);
	view->scalex *= superg->scalex;
	view->scaley *= superg->scaley;
	view->rotation += superg->rotation;
        memcpy(&view->cliprect, &cliprect, sizeof(NXRect));
    }

    return view;
}

- _displayFreeGState
{
    if (vFlags.validGState) {
        XMotifContext context;
	XWgstate view = (XWgstate)_gstate;

        context = (XMotifContext)[NXApp context];
	XFreeGC(context->display, view->xgstate);
	NX_FREE(view);
    }
    vFlags.validGState = 0;
    return self;
}

/* This should only get called when we don't already have a gstate, but 
   want one. We initialize a stock gstate, i.e. no scaling, rotation, etc.
*/
- (void *)_displayGState
{
    [self _createGState];
    if (vFlags2.notifyToInitGState == YES)
	[self initGState];
    return self;
}

- (BOOL)_displayLockFocus
{
    XMotifContext context;

    context = (XMotifContext)[NXApp context];

    // Push the current gstate
    if (context->gstate) {
	[(List *)context->gstack addObject:(id)context->gstate];
	// the current gstate should be our superview's gstate
	if (superview && context->gstate->widget != [superview _widget]) {
	    fprintf(stderr, "Error (View): Focus already locked on different view.\n");
	    //return NO;
	}
    }
    
    // Install our own gstate if we have one, otherwise create a temporary one
    // Always create a gstate for a top-level view (done in _createGState)
    if (!_gstate) {
	XWgstate newGState;
    	newGState = [self _createGState];
	if (context->gstate)
    	    context->gstate = [self _updateGState:newGState 
					fromGState:context->gstate];
	else
	    context->gstate = newGState;
    } else
    	context->gstate = (XWgstate)_gstate;

    return YES;
}

- _displayUnlockFocus
{
    XMotifContext context;

    context = (XMotifContext)[NXApp context];
    if (!context->gstate) {
	fprintf(stderr, "Error (View): attempt to unlock a nil view.\n");
	return nil;
    }
    if (context->gstate->widget != [self _widget]) {
	fprintf(stderr,
		"Error (View): attempt to unlock a view that was not locked\n");
	//return nil;
    }

    // If we are using backing store, we need to copy the data to the window
    if (context->gstate->update)
    	XWPing(context, frame);
    
    // Free our temporary gstate
    if (!_gstate)
	NX_FREE(context->gstate);

    context->gstate = (XWgstate)[(List *)context->gstack removeLastObject];
    return self;
}

- _setSize:(NXSize *)newSize
{
    [self _setArg:XmNwidth  to:(int)newSize->width];
    [self _setArg:XmNheight to:(int)newSize->height];
    frame.size.width = newSize->width;
    frame.size.height = newSize->height;
    return self;
}

/* these last three methods set x resources, and should be methods only known
   to the category extension.  Box makes use of all of these methods in
   its content view.
 */
- _setResize:(void *)policy
{
    [self _setArg:XmNresizePolicy  to:(XtArgVal)policy];
    return self;
}

- _setOffsets:(NXSize *)offsets
{
    [self _setArg:XmNmarginWidth   to:offsets->width];
    [self _setArg:XmNmarginHeight  to:offsets->height];
    return self;
}

- _setBorderShadow:(void *)shadow andThickness:(void *)thickness
{
    [self _setArg:XmNshadowType      to:(XtArgVal)shadow];
    [self _setArg:XmNshadowThickness to:(XtArgVal)thickness];
    return self;
}

- _setFrame:(const NXRect *)aFrame
{
    [self _setArg:XmNx      to:(int)aFrame->origin.x];
    [self _setArg:XmNy      to:(int)aFrame->origin.y];
    [self _setArg:XmNwidth  to:(int)aFrame->size.width];
    [self _setArg:XmNheight to:(int)aFrame->size.height];
    return self;
}

@end


// Create a default graphics context.
GC
_createGC(XWContext context, Widget widget) 
{
    GC	      gc;
    XGCValues gcv;

    if (!widget) {
	fprintf(stderr, "Error (View): Attempt to create a GC on an unrealized View\n");
	return NULL;
    }

    gcv.function = GXcopy;
    gcv.foreground = xtGrayToPixel(context, 0.0);	/* NX_BLACK */
    gcv.background = xtGrayToPixel(context, 0.66666);	/* NX_LTGRAY */
    gcv.plane_mask = AllPlanes;
    gc = XCreateGC(context->display, XtWindow(widget),
    	GCFunction | GCForeground | GCBackground | GCPlaneMask , &gcv);
    return gc;
}

// Copy a graphic context
GC
_copyGC(XWContext context, Widget widget, GC source) 
{
    GC	      		dest;
    unsigned long	mask;

    mask = 0xffffffff; /* Copy everything (Hopefully) */
    dest = XCreateGC(context->display, XtWindow(widget), 0, NULL);
    XCopyGC(context->display, source, mask, dest); 
    return dest;
}

