/* from
 * (Preliminary Documentation) Copyright (c) 1994 by NeXT Computer, Inc.  
 * All Rights Reserved.
 *
 * part of Foundation Kit
 *
 * NSArchiver 
 *
 */
#import <foundation/NSArchiver.h>
 
@implementation NSArchiver

// Initializing an NSArchiver

- (id)initForWritingWithMutableData:(NSMutableData *)mdata
 /*
  * Initializes an archiver, encoding stream and version information into
  * mutable data mdata. 
  */
{
    theData = mdata;
    return self;
}

// Archiving data 

+ (NSData *)archivedDataWithRootObject:(id)rootObject
 /*
  * Creates and returns a data object after initializing an archiver with
  * that object and encoding the archiver with rootObject. 
  */
{
    NSArchiver		*anArchiver;
    NSMutableData	*aData;
    
    self = [NSArchiver alloc];
    aData = [NSData alloc];
    [self initForWritingWithMutableData:aData];
    [self encodeRootObject:rootObject];
    return [self archiverData];
}

+ (BOOL)archiveRootObject:(id)rootObject
                   toFile:(NSString *)path
 /*
  * Archives rootObject by encoding it as a data object in an archiver and
  * writing that data object to file path.  Returns YES upon success. 
  */
{
    theData = [self archiveDataWithRootObject:rootObject];
    return [theData writeToFile:path];
}

- (void)encodeArrayOfObjCType:(const char *)type
                        count:(unsigned int)count
			   at:(const void *)array
 /*
  * Encodes an array of count data elements of the same   Objective C data
  * type.  
  */
{
    [self notImplemented:_cmd];
    return;
}

- (void)encodeConditionalObject:(id)object
 /*
  * Encodes into the linearized data a conditional  object that points back
  * toward a root object.  If nil is specified for object, it encodes it as
  * nil unconditionally.  Raises an exception if no root object has been
  * encoded.    
  */
{
    [self notImplemented:_cmd];
    return;
}

- (void)encodeRootObject:(id)rootObject
 /*
  * Encodes the rootObject at the start of the linearized data representing
  * the object graph.  Raises an exception if the root object has already
  * been encoded. 
  */
{
    id	callback = nil;
    
    if ( [rootObject conformsTo:@protocol(NSObjCTypeSerializationCallBack)] ) {
	callback = rootObject;
    }
    [theData serializeDataAt:rootObject
                  ofObjCType:@encode(rootObject)
	             context:callback];
    return;
}

// Getting data from the archiver

- (NSMutableData *)archiverData;
 /*
  * Returns the data object,  in mutable form, that is associated with the
  * receiving NSArchiver. 
  */
{
    return theData;
}

// Substituting One Class for Another 

+ (NSString *)classNameEncodedForTrueClassName:(NSString *)trueName;
 /*
  * Returns the string object representing the class name used to archive
  * instances of the class.  This class name might not be the original name
  * (trueName):  see encodeClassName:intoClassName. 
  */
{
    [self notImplemented:_cmd];
    return;
}

- (void)encodeClassName:(NSString *)trueName
	  intoClassName:(NSString *)inArchiveName;
 /*
  * Encodes in the archived data a substitute class name (inArchiveName) for
  * the real class name (trueName).  
  */
{
    [self notImplemented:_cmd];
    return;
}


@end
