/* From
 * (Preliminary Documentation) Copyright (c) 1994 by NeXT Computer, Inc.
 *  All Rights Reserved.
 *
 * NSValue 
 */

#ifndef __NSValue_INCLUDE_GNU
#define __NSValue_INCLUDE_GNU

#include "NSObject.h"

#include "NSGeometry.h"

@interface NSValue : NSObject<NSCopying>
{
    void		*_dataptr;
    unsigned int	_size;
    char		*objctype;
}

// Allocating and Initializing 

+ (NSValue *)value:(const void *)value
      withObjCType:(const char *)type;
 /*
  * Creates and returns a value object containing the value  value of the
  * Objective C type type. 
  */
		
+ (NSValue *)valueWithNonretainedObject: (id)anObject;
 /*
  * Creates and returns a value object containing the object anObject,
  * without retaining anObject. 
  */
	
+ (NSValue *)valueWithPoint:(NSPoint)point;
 /*
  * Creates and returns a value object that contains the specified NSPoint
  * structure (which represents a geometrical point in two dimensions). 
  */
 
+ (NSValue *)valueWithPointer:(const void *)pointer;
 /*
  * Creates and returns a value object that contains the specified pointer. 
  */
	
+ (NSValue *)valueWithRect:(NSRect)rect;
 /*
  * Creates and returns a value object that contains the specified NSRect
  * structure, representing a rectangle. 
  */
 
+ (NSValue *)valueWithSize:(NSSize)size;
 /*
  * Creates and returns a value object that contains the specified NSSize
  * structure (which stores a width and a height). 
  */


// Accessing Data 

- (void)getValue:(void *)value;
 /*
  * Copies the receiver's data into value. 
  */
 
- (id)nonretainedObjectValue;
 /*
  * Returns the non-retained object that's contained in the receiver. 
  */
 
- (const char *)objCType;
 /*
  * Returns the Objective C type of the data contained in the receiver. 
  */
 
- (void *)pointerValue;
 /*
  * Returns the pointer that's contained in the receiver. 
  */
 
- (NSRect)rectValue;
 /*
  * Returns the rectangle structure that's contained in the receiver. 
  */
 
- (NSSize)sizeValue;
 /*
  * Returns the size structure that's contained in the receiver. 
  */
 
- (NSPoint)pointValue;
 /*
  * Returns the point structure that's contained in the receiver. 
  */


@end

#endif
