/* From
 * (Preliminary Documentation) Copyright (c) 1994 by NeXT Computer, Inc.
 *  All Rights Reserved.
 *
 * NSValue 
 */

#include "NSValue.h"

#include <objc/encoding.h>
#include <stdlib.h>
#include <string.h>

@implementation NSValue

// protocol conformance

- (id)copyWithZone:(NSZone *)zone
{
    return [self copy];
}

// Allocating and Initializing 

+ (NSValue *)value:(const void *)value
      withObjCType:(const char *)type
{
    NSValue		*newValue;
    unsigned int	typelen;
    
    newValue = [self alloc];
    typelen = strlen(type);
    newValue->objctype = (char *)malloc( typelen +1 );
    memcpy( newValue->objctype, type, typelen+1);
    newValue->_size = objc_sizeof_type(type);
    newValue->_dataptr = (void *)malloc(newValue->_size);
    memcpy( newValue->_dataptr, value, newValue->_size);
    return newValue;
}
		
+ (NSValue *)valueWithNonretainedObject: (id)anObject
{
    id		copiedObject;
        
    copiedObject = [anObject copy];
    return [self value:copiedObject withObjCType:"@"];
}
	
+ (NSValue *)valueWithPoint:(NSPoint)point
{
    return [self value:&point withObjCType:@encode(NSPoint)];
}
 
+ (NSValue *)valueWithPointer:(const void *)pointer
{
    return [self value:&pointer withObjCType:@encode(void *)];
}

+ (NSValue *)valueWithRect:(NSRect)rect
{
    return [self value:&rect withObjCType:@encode(NSRect)];
}
 
+ (NSValue *)valueWithSize:(NSSize)size
 /*
  * Creates and returns a value object that contains the specified NSSize
  * structure (which stores a width and a height). 
  */
{
    return [self value:&size withObjCType:@encode(NSSize)];
}


// Accessing Data 

- (void)getValue:(void *)value
 /*
  * Copies the receiver's data into value. 
  */
{
    memcpy( value, _dataptr, _size );
    return;
}

- (id)nonretainedObjectValue
{
    return (id) _dataptr;
}
 
- (const char *)objCType
{
    return objctype;
}
 
- (void *)pointerValue
{
    void *pointer;
    
    [self getValue:&pointer];
    return pointer;
} 

- (NSRect)rectValue
{
    NSRect value;
    
    [self getValue:&value];
    return value;
}
 
- (NSSize)sizeValue
{
    NSSize size;
    
    [self getValue:&size];
    return size;
}
 
- (NSPoint)pointValue
{
    NSPoint point;

    [self getValue:&point]; 
    return point;
}

@end
